package ch.unige.solidify.util;

import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

import ch.unige.solidify.service.HttpRequestInfoProvider;

/**
 * This RestClientTool use the token from the incoming request, if it exists, otherwise it fallback to the trusted token.
 */
@Profile({"client-backend", "test"})
@Component
public class FallbackRestClientTool extends AbstractRestClientTool {

  private final HttpRequestInfoProvider httpRequestInfoProvider;
  private String trustedToken;

  public FallbackRestClientTool(HttpRequestInfoProvider httpRequestInfoProvider) {
    this.httpRequestInfoProvider = httpRequestInfoProvider;
  }

  @Override
  protected RestTemplateBuilder addToken(RestTemplateBuilder restTemplateBuilder) {
    if(httpRequestInfoProvider.getIncomingToken() != null) {
      return this.addTokenFromRequest(restTemplateBuilder, this.httpRequestInfoProvider);
    } else {
      return this.addTrustedToken(restTemplateBuilder, this.trustedToken);
    }
  }

  public void setTrustedToken(String trustedToken) {
    this.trustedToken = trustedToken;
  }

  public boolean hasTrustedToken() {
    return !StringTool.isNullOrEmpty(this.trustedToken);
  }
}
