package ch.unige.solidify.util;

import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

/**
 * This RestClientTool use the tokens obtained by the trusted users from the authorization server.
 * This is useful for inter-modules REST call not initiated by a user, or when elevating the user privileges is necessary to perform an action.
 */
@Profile({"client-backend", "test"})
@Component
public class TrustedRestClientTool extends AbstractRestClientTool {

  private String trustedToken;

  @Override
  protected RestTemplateBuilder addToken(RestTemplateBuilder restTemplateBuilder) {
    return this.addTrustedToken(restTemplateBuilder, this.trustedToken);
  }

  public void setTrustedToken(String trustedToken) {
    this.trustedToken = trustedToken;
  }

  public boolean hasTrustedToken() {
    return !StringTool.isNullOrEmpty(this.trustedToken);
  }

}
