package ch.unige.solidify.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.config.SolidifyProperties;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.security.UserPermissions;
import ch.unige.solidify.service.GitInfoProperties;
import ch.unige.solidify.service.ModuleService;

@UserPermissions
@Controller
public class DashboardController {

  private final SolidifyProperties config;
  private final GitInfoProperties gitInfoProperties;
  private final ModuleService<? extends Resource> moduleService;

  public DashboardController(SolidifyProperties config, GitInfoProperties gitInfoProperties,
          ModuleService<? extends Resource> moduleService) {
    this.config = config;
    this.gitInfoProperties = gitInfoProperties;
    this.moduleService = moduleService;
  }

  @GetMapping(SolidifyConstants.URL_DASHBOARD)
  public String dashboard(Model model) {
    model.addAttribute("applicationName", this.config.getServerDisplayName());
    model.addAttribute("applicationVersion", this.gitInfoProperties.getBuild().getVersion());
    model.addAttribute("modules", this.moduleService.getModuleControllerList());
    return "dashboard";
  }
}
