package ch.unige.solidify.service;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.TargetClassAware;
import org.springframework.aop.framework.Advised;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import ch.unige.solidify.controller.ControllerWithHateoasHome;
import ch.unige.solidify.controller.ModuleController;
import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.Resource;

@Service
public class ModuleService<T extends Resource> implements ApplicationContextAware {
  private static final Logger log = LoggerFactory.getLogger(ModuleService.class);
  private final List<ModuleController> moduleControllerList = new ArrayList<>();

  public List<ModuleController> getModuleControllerList() {
    return this.moduleControllerList;
  }

  @Override
  public void setApplicationContext(ApplicationContext applicationContext) {

    final String[] modList = applicationContext.getBeanNamesForType(ModuleController.class);
    log.info("Modules found : {}", modList.length);
    for (final String b : modList) {
      log.info("Finding module: {}", b);
      this.moduleControllerList.add((ModuleController) applicationContext.getBean(b));
    }
    final String[] resList = applicationContext.getBeanNamesForType(ControllerWithHateoasHome.class);
    for (final String r : resList) {
      log.info("Finding resource: {}", r);
      if (!this.addResource((ControllerWithHateoasHome) applicationContext.getBean(r))) {
        log.error("Resource ( {} ) unassigned to a module", r);
      }
    }
  }

  private boolean addResource(ControllerWithHateoasHome resource) {

    final String name = resource.getClass().getName();
    final String module = name.substring(0, name.lastIndexOf('.')).toLowerCase();
    for (final ModuleController mod : this.moduleControllerList) {
      if (mod.getClass().getCanonicalName().toLowerCase().startsWith(module)) {
        if (log.isDebugEnabled()) {
          log.debug("Handling resource controller {}", resource);
        }
        if (resource instanceof TargetClassAware) {
          try {
            mod.addResource(
                    (ControllerWithHateoasHome) ((Advised) resource).getTargetSource().getTarget());
          } catch (final Exception e) {
            throw new SolidifyRuntimeException("Unproxy object resource rejection. Resource: ["
                    + ((Advised) resource).getTargetClass() + "]", e);
          }
        } else {
          mod.addResource(resource);
        }

        return true;
      }
    }
    return false;
  }

}