package ch.unige.solidify.service;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.SearchCondition;

public abstract class NoSqlResourceService<T> extends AbstractResourceService {

  public Page<T> findAll(T search, Pageable pageable) {
    final List<T> list = this.findAll(search);
    final long start = pageable.getOffset();
    final long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  // Default Functions
  public Page<T> search(String query, Pageable pageable) {
    return this.search(query, (String) null, null, null, null, pageable);
  }

  public FacetPage<T> search(String query, String conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    Page page = this.search(query, (String) null, null, null, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> search(String query, String set, String field,
          String from, String to, Pageable pageable) {
    return this.findAll(null, pageable);
  }

  public FacetPage<T> search(String query, List<FacetRequest> facetRequests, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    Page page = this.findAll(null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> searchRange(String field, String from, String to,
          Pageable pageable) {
    return this.findAll(null, pageable);
  }

  public FacetPage<T> search(List<SearchCondition> conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    return this.search( null, null, facetRequests, pageable);
  }

  public abstract T findOne(String id);

  public abstract List<T> findAll(T search);

  public abstract T save(T t);

  public abstract T update(T t);

  public abstract boolean delete(T t);

}
