package ch.unige.solidify.service;

import org.springframework.data.domain.Pageable;

import ch.unige.solidify.exception.SolidifyRuntimeException;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.ResourceClient;
import ch.unige.solidify.util.StringTool;

public abstract class RemoteResourceService<T extends Resource> implements ResourceClient<T> {

  protected SolidifyRestClientService restClientService;

  public RemoteResourceService(SolidifyRestClientService restClientService) {
    this.restClientService = restClientService;
  }

  /**
   * Retrieves a list of all RESTResource by sending a REST request to the corresponding webservice
   *
   * @param pageable
   * @return
   */
  @Override
  public RestCollection<T> findAll(Pageable pageable) {
    return this.restClientService.getResourceList(this.getResourceUrl(), this.getResourceClass(), pageable);
  }

  /**
   * Retrieves a RESTResource by sending a REST request to the corresponding webservice
   *
   * @param resId The id of the RESTResource to fetch
   * @return
   */
  @Override
  public T findOne(String resId) {
    if (!StringTool.isNullOrEmpty(resId)) {
      final String url = this.getResourceUrl() + "/" + resId;
      return this.restClientService.getResource(url, this.getResourceClass());
    } else {
      throw new SolidifyRuntimeException("resId should not be null nor empty");
    }
  }

  protected abstract Class<T> getResourceClass();

  protected String getResourceUrl() {
    throw new UnsupportedOperationException(
            "Missing resource URL for type " + this.getClass().getName());
  }
}
