package ch.unige.solidify.controller;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.security.RootPermissions;
import ch.unige.solidify.service.IndexResourceService;

@RootPermissions
public abstract class IndexResourceController<I, T extends IndexMetadata>
        extends IndexResourceReadOnlyController<I, T> implements ControllerWithHateoasHome {

  public IndexResourceController(IndexResourceService<I, T> indexResourceService) {
    super(indexResourceService);
  }

  @PostMapping
  public HttpEntity<T> create(@RequestBody T t) {
    final T item = this.indexResourceService.save(this.getIndex(), t);
    return new ResponseEntity<>(item, HttpStatus.CREATED);
  }

  @DeleteMapping(SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String id) {
    final T t = this.indexResourceService.findOne(this.getIndex(), id);
    if (t == null) {
      throw new SolidifyHttpErrorException(HttpStatus.NOT_FOUND, "Item " + id + " not found");
    }
    if (!this.indexResourceService.delete(this.getIndex(), t)) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, "Resource " + id + " gone");
    }
    return HttpStatus.OK;
  }

  @DeleteMapping
  public HttpStatus deleteList(@RequestBody String[] ids) {
    // Remove each item
    for (final String id : ids) {
      final T t = this.indexResourceService.findOne(this.getIndex(), id);
      if (t == null) {
        throw new SolidifyHttpErrorException(HttpStatus.NOT_FOUND, "Parent item " + id + " not found");
      }
      this.indexResourceService.delete(this.getIndex(), t);
    }
    return HttpStatus.OK;
  }

  @PatchMapping(SolidifyConstants.URL_ID)
  public HttpEntity<T> update(@PathVariable String id, @RequestBody T t2) {
    final T t1 = this.indexResourceService.findOne(this.getIndex(), id);
    if (t1 == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    if (t1.update(t2)) {
      final T item = this.indexResourceService.update(this.getIndex(), t2);
      return new ResponseEntity<>(item, HttpStatus.OK);
    }
    return new ResponseEntity<>(t1, HttpStatus.OK);
  }

}
