package ch.unige.solidify.service;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.SearchCondition;

public abstract class IndexResourceService<I, T extends IndexMetadata> extends AbstractResourceService {

  public Page<T> findAll(I indexName, T search, Pageable pageable) {
    final List<T> list = this.findAll(indexName, search);
    final long start = pageable.getOffset();
    final long end = (start + pageable.getPageSize()) > list.size() ? list.size()
            : (start + pageable.getPageSize());
    return new PageImpl<>(list.subList((int) start, (int) end), pageable, list.size());
  }

  // Default Functions
  public Page<T> search(I indexName, String query, Pageable pageable) {
    return this.search(indexName, query, (String) null, null, null, null, pageable);
  }

  public FacetPage<T> search(I indexName, String query, String conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    Page page = this.search(indexName, query, (String) null, null, null, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> search(I indexName, String query, String set, String field,
          String from, String to, Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public FacetPage<T> search(I indexName, String query, List<FacetRequest> facetRequests, String set, String field,
          String from, String to, String filter, Pageable pageable) {
    Page page = this.findAll(indexName, null, pageable);

    if (page instanceof FacetPage) {
      return (FacetPage<T>) page;
    } else {
      return new FacetPage<T>(page.getContent(), null, page.getPageable(), page.getTotalElements());
    }
  }

  public Page<T> searchRange(I indexName, String field, String from, String to,
          Pageable pageable) {
    return this.findAll(indexName, null, pageable);
  }

  public FacetPage<T> search(I indexName, List<SearchCondition> conditions, List<FacetRequest> facetRequests, Pageable pageable) {
    return this.search(indexName, null, null, facetRequests, pageable);
  }

  public abstract T findOne(I indexName, String id);

  public abstract List<T> findAll(I indexName, T search);

  public abstract T save(I indexName, T t);

  public abstract T update(I indexName, T t);

  public abstract boolean delete(I indexName, T t);

  // Define Database name ~ index name
  public abstract String getIndexName(I indexName);
}
