package ch.unige.solidify.service;

import java.util.HashMap;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.rest.FacetPage;
import ch.unige.solidify.rest.FacetRequest;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.service.rest.abstractservice.IndexMetadataRemoteResourceService;
import ch.unige.solidify.util.CollectionTool;

public abstract class IndexSearchService<I, T extends IndexMetadata> extends IndexResourceService<I, T> {

  private final IndexMetadataRemoteResourceService<I, T> indexMetadataRemoteResourceService;

  public IndexSearchService(IndexMetadataRemoteResourceService<I, T> indexMetadataRemoteResourceService) {
    this.indexMetadataRemoteResourceService = indexMetadataRemoteResourceService;
  }

  @Override
  public List<T> findAll(I indexName, T search) {
    HashMap<String, Object> parameters = new HashMap<>();
    parameters.put(IndexConstants.SEARCH_QUERY, "*");
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataList(
            indexName,
            parameters,
            PageRequest.of(0, RestCollectionPage.MAX_SIZE_PAGE));
    return CollectionTool.toList(result.get_data());
  }

  @Override
  public Page<T> findAll(I indexName, T search, Pageable pageable) {
    HashMap<String, Object> parameters = new HashMap<>();
    parameters.put(IndexConstants.SEARCH_QUERY, "*");
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataList(
            indexName,
            parameters,
            pageable);
    return new PageImpl<>(CollectionTool.toList(result.get_data()), pageable, result.get_page().getTotalItems());
  }

  @Override
  public T findOne(I indexName, String id) {
    return this.indexMetadataRemoteResourceService.getIndexMetadata(indexName, id);
  }

  @Override
  public FacetPage<T> search(I indexName, String query, String conditions, List<FacetRequest> facetRequests,
          Pageable pageable) {
    final RestCollection<T> result = this.indexMetadataRemoteResourceService.getIndexMetadataListWithConditions(
            indexName,
            query,
            conditions,
            pageable);
    return new FacetPage<>(CollectionTool.toList(result.get_data()), result.get_facets(), pageable, result.get_page().getTotalItems());
  }

  @Override
  public T save(I indexName, T t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public T update(I indexName, T t) {
    throw new UnsupportedOperationException();
  }

  @Override
  public boolean delete(I indexName, T t) {
    throw new UnsupportedOperationException();
  }
}
