package ch.unige.solidify.service.rest.abstractservice;

import java.util.ArrayList;
import java.util.List;

import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.dto.LabelDTO;
import ch.unige.solidify.model.index.FacetProperties;
import ch.unige.solidify.model.index.IndexFieldAlias;
import ch.unige.solidify.model.index.IndexFieldAliasLabel;
import ch.unige.solidify.rest.RestCollectionPage;
import ch.unige.solidify.service.RemoteResourceService;
import ch.unige.solidify.service.SolidifyRestClientService;
import ch.unige.solidify.util.StringTool;

public abstract class IndexFielAliasRemoteResourceService extends RemoteResourceService<IndexFieldAlias> {

  private final String indexUrl;

  protected IndexFielAliasRemoteResourceService(String rootContext, SolidifyRestClientService restClientService) {
    super(restClientService);
    this.indexUrl = rootContext + SolidifyConstants.URL_SEP + IndexConstants.INDEX_MODULE;
  }

  @Override
  protected Class<IndexFieldAlias> getResourceClass() {
    return IndexFieldAlias.class;
  }

  @Override
  protected String getResourceUrl() {
    return this.indexUrl + SolidifyConstants.URL_SEP + IndexConstants.INDEX_FIELD_ALIASES;
  }

  public List<FacetProperties> getFacetProperties(String index) {
    List<FacetProperties> list = new ArrayList<>();
    for (final IndexFieldAlias field : this
            .findAll(PageRequest.of(0, RestCollectionPage.MAX_SIZE_PAGE, Sort.by(IndexConstants.FACET_ORDER_FIELD))).get_data()) {
      if (field.getIndexName().equals(index)) {
        FacetProperties facetProperties = new FacetProperties();
        facetProperties.setName(field.getAlias());
        facetProperties.setDefaultVisibleValues(field.getFacetDefaultVisibleValues());
        for (IndexFieldAliasLabel label : field.getLabels()) {
          String text = label.getText();
          if (StringTool.isNullOrEmpty(text)) {
            text = field.getAlias();
          }
          facetProperties.getLabels().add(new LabelDTO(text, label.getLanguage().getCode()));
        }
        list.add(facetProperties);
      }
    }
    return list;
  }

}
