package ch.unige.solidify.model.index;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ResourceNormalized;
import ch.unige.solidify.util.ReflectionTool;

@Entity
@Table(uniqueConstraints = { @UniqueConstraint(columnNames = { "indexName", "alias" }) })
public class IndexFieldAlias extends ResourceNormalized {

  /**
   * The name of the ElasticSearch index the field is for
   */
  @NotNull
  @Size(min = 1)
  private String indexName;

  /**
   * The name that will be exposed to clients (as a field or a facet)
   */
  @NotNull
  @Size(min = 1)
  private String alias;

  /**
   * The field path in the index
   */
  @NotNull
  @Size(min = 1)
  private String field;

  /**
   * Should a facet be created for this field ? default to false
   */
  @NotNull
  private Boolean facet;

  /**
   * Minimum number of occurrences of a facet value to be returned in facet results if the
   * IndexFieldAlias is a facet
   */
  private Integer facetMinCount;

  /**
   * Maximum number of facets values to return if the IndexFieldAlias is a facet
   */
  private Integer facetLimit;

  /**
   * Determines the position of a facet in the user interface
   */
  private Integer facetOrder;

  /**
   * Number of facet values displayed by default in the user interface
   */
  private Integer facetDefaultVisibleValues;

  @Valid
  @ElementCollection
  @Column(name = IndexConstants.DB_INDEX_FIELD_ALIAS_ID)
  @CollectionTable(joinColumns = {
          @JoinColumn(name = IndexConstants.DB_INDEX_FIELD_ALIAS_ID) }, uniqueConstraints = @UniqueConstraint(columnNames = {
                  IndexConstants.DB_INDEX_FIELD_ALIAS_ID, SolidifyConstants.DB_LANGUAGE_ID }))
  private List<IndexFieldAliasLabel> labels = new ArrayList<>();

  public String getIndexName() {
    return this.indexName;
  }

  public void setIndexName(String indexName) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), indexName);
  }

  public String getAlias() {
    return this.alias;
  }

  public void setAlias(String alias) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), alias);
  }

  public String getField() {
    return this.field;
  }

  public void setField(String field) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), field);
  }

  public Boolean isFacet() {
    return this.facet;
  }

  public void setFacet(Boolean facet) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), facet);
  }

  public Integer getFacetMinCount() {
    return this.facetMinCount;
  }

  public void setFacetMinCount(Integer minCount) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), minCount);
  }

  public Integer getFacetLimit() {
    return this.facetLimit;
  }

  public void setFacetLimit(Integer limit) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), limit);
  }

  public Integer getFacetOrder() {
    return this.facetOrder;
  }

  public void setFacetOrder(Integer facetOrder) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), facetOrder);
  }

  public Integer getFacetDefaultVisibleValues() {
    return this.facetDefaultVisibleValues;
  }

  public void setFacetDefaultVisibleValues(Integer facetDefaultVisibleValues) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), facetDefaultVisibleValues);
  }

  public List<IndexFieldAliasLabel> getLabels() {
    return this.labels;
  }

  public void setLabels(List<IndexFieldAliasLabel> labels) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), labels);
  }

  @Override
  public void init() {
    if (this.isFacet() == null) {
      this.setFacet(false);
    }
  }

  @Override
  public String managedBy() {
    return IndexConstants.INDEX_MODULE;
  }

}
