package ch.unige.solidify.model.index;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonNode;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyValidationException;
import ch.unige.solidify.validation.ValidationError;

public abstract class IndexMetadataInfoDeserializer<T extends IndexMetadata> extends JsonDeserializer<T> {

  protected abstract T getNewObject();

  @Override
  public T deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException {
    final JsonNode node = jp.getCodec().readTree(jp);
    if (node.has("index") && node.has(SolidifyConstants.DB_RES_ID)) {
      final T indexMetadata = this.getNewObject();
      indexMetadata.setResId(node.get(SolidifyConstants.DB_RES_ID).asText());
      indexMetadata.setIndex(node.get("index").asText());
      if (node.has("metadata")) {
        indexMetadata.setMetadata(node.findValue("metadata").toString());
      }
      return indexMetadata;
    } else {
      throw new SolidifyValidationException(
              new ValidationError("JSON fields 'index' and '" + SolidifyConstants.DB_RES_ID + "' must be present"));
    }
  }
}
