package ch.unige.solidify.model.specification;

import java.util.List;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.model.index.IndexFieldAlias;
import ch.unige.solidify.specification.SolidifySpecification;

public class IndexFieldAliasSpecification extends SolidifySpecification<IndexFieldAlias> {

  private static final long serialVersionUID = 6166149531574712100L;

  public IndexFieldAliasSpecification(IndexFieldAlias criteria) {
    super(criteria);
  }

  @Override
  protected void completePredicatesList(Root<IndexFieldAlias> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {
    if (this.criteria.getIndexName() != null) {
      predicatesList.add(builder.equal(root.get("indexName"), this.criteria.getIndexName()));
    }

    if (this.criteria.getAlias() != null) {
      predicatesList.add(builder.like(root.get("alias"), "%" + this.criteria.getAlias() + "%"));
    }

    if (this.criteria.getField() != null) {
      predicatesList.add(builder.like(root.get("field"), "%" + this.criteria.getField() + "%"));
    }

    if (this.criteria.isFacet() != null) {
      predicatesList.add(builder.equal(root.get("facet"), this.criteria.isFacet()));
    }

    if (this.criteria.getFacetMinCount() != null) {
      predicatesList.add(builder.equal(root.get("facetMinCount"), this.criteria.getFacetMinCount()));
    }

    if (this.criteria.getFacetLimit() != null) {
      predicatesList.add(builder.equal(root.get("facetLimit"), this.criteria.getFacetLimit()));
    }

    if (this.criteria.getFacetOrder() != null) {
      predicatesList.add(builder.equal(root.get("facetOrder"), this.criteria.getFacetOrder()));
    }

    if (this.criteria.getFacetDefaultVisibleValues() != null) {
      predicatesList.add(builder.equal(root.get("facetDefaultVisibleValues"), this.criteria.getFacetDefaultVisibleValues()));
    }
  }
}
