package ch.unige.solidify.controller.index;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.IndexConstants;
import ch.unige.solidify.IndexProperties;
import ch.unige.solidify.IndexUrlPath;
import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.IndexController;
import ch.unige.solidify.controller.NoSqlResourceController;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.index.settings.IndexingSettingsService;
import ch.unige.solidify.model.index.IndexSettings;
import ch.unige.solidify.rest.RestCollection;
import ch.unige.solidify.security.RootPermissions;

@RootPermissions
@RestController
@ConditionalOnBean(IndexController.class)
@RequestMapping(IndexUrlPath.INDEX_SETTING)
public class IndexSettingsController extends NoSqlResourceController<IndexSettings> {
  private static final Logger log = LoggerFactory.getLogger(IndexSettingsController.class);

  public IndexSettingsController(IndexingSettingsService<IndexSettings> noSqlResourceService, IndexProperties indexProperties) {
    super(noSqlResourceService);
  }

  @Override
  public HttpEntity<IndexSettings> create(@RequestBody IndexSettings archiveIndex) {
    return super.create(archiveIndex);
  }

  @Override
  public HttpEntity<IndexSettings> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  public HttpEntity<RestCollection<IndexSettings>> list(@ModelAttribute IndexSettings search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpEntity<IndexSettings> update(@PathVariable String id, @RequestBody IndexSettings archiveIndex) {
    return super.update(id, archiveIndex);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @DeleteMapping(SolidifyConstants.URL_SEP + IndexConstants.DELETE_ALL)
  public HttpStatus deleteAll(@RequestParam(defaultValue = "no") String confirm) {
    final List<IndexSettings> list = this.noSqlResourceService.findAll(null);
    if (list.isEmpty()) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, " gone");
    }
    for (final IndexSettings i : list) {
      if (confirm.equals(SolidifyConstants.YES)) {
        if (!this.noSqlResourceService.delete(i)) {
          log.error(this.messageService.get("data-mgmt.index.delete.error", new Object[] { i.getResId() }));
          throw new SolidifyHttpErrorException(HttpStatus.INTERNAL_SERVER_ERROR, "Cannot delete index " + i.getResId());
        }
        log.info(this.messageService.get("data-mgmt.index.delete.info", new Object[] { i.getResId() }));
      } else {
        log.info(this.messageService.get("data-mgmt.index.delete.preview", new Object[] { i.getResId() }));
      }
    }
    return HttpStatus.OK;
  }

}
