package ch.unige.solidify.index;

import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import ch.unige.solidify.IndexProperties;
import ch.unige.solidify.controller.IndexController;

@Profile("index-elasticsearch")
@Service
@ConditionalOnBean(IndexController.class)
public class ElasticsearchClientProvider {

  private final RestHighLevelClient elasticsearchClient;

  public ElasticsearchClientProvider(IndexProperties indexProperties) {

    final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
    credentialsProvider.setCredentials(AuthScope.ANY,
            new UsernamePasswordCredentials(indexProperties.getConfig().getUsername(), indexProperties.getConfig().getPassword()));

    String scheme = "http";
    if (indexProperties.getConfig().isSslEnabled()) {
      scheme = "https";
    }
    HttpHost host = new HttpHost(indexProperties.getConfig().getHost(), indexProperties.getConfig().getPort(), scheme);
    RestClientBuilder restClientBuilder = RestClient.builder(host);
    restClientBuilder.setHttpClientConfigCallback(httpClientBuilder -> httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider));

    this.elasticsearchClient = new RestHighLevelClient(restClientBuilder);

  }

  public RestHighLevelClient getElasticsearchClient() {
    return this.elasticsearchClient;
  }

}
