package ch.unige.solidify.service;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.stereotype.Service;

import ch.unige.solidify.controller.IndexController;
import ch.unige.solidify.index.indexing.IndexingService;
import ch.unige.solidify.model.index.IndexMetadata;
import ch.unige.solidify.model.index.IndexSettings;

@Service
@ConditionalOnBean(IndexController.class)
public class IndexSettingIndexMetadataService extends IndexResourceService<IndexSettings, IndexMetadata> {

  private IndexingService<IndexMetadata> indexingService;

  public IndexSettingIndexMetadataService(IndexingService<IndexMetadata> indexingService) {
    this.indexingService = indexingService;
  }

  @Override
  public IndexMetadata findOne(IndexSettings indexName, String id) {
    return this.indexingService.findOne(this.getIndexName(indexName), id);
  }

  @Override
  public List<IndexMetadata> findAll(IndexSettings indexName, IndexMetadata search) {
    return this.indexingService.findAll(this.getIndexName(indexName), search);
  }

  @Override
  public IndexMetadata save(IndexSettings indexName, IndexMetadata indexMetadata) {
    return this.indexingService.save(this.getIndexName(indexName), indexMetadata);
  }

  @Override
  public IndexMetadata update(IndexSettings indexName, IndexMetadata indexMetadata) {
    indexMetadata.setIndex(this.getIndexName(indexName));
    return this.indexingService.update(this.getIndexName(indexName), indexMetadata);
  }

  @Override
  public boolean delete(IndexSettings indexName, IndexMetadata indexMetadata) {
    indexMetadata.setIndex(this.getIndexName(indexName));
    return this.indexingService.delete(this.getIndexName(indexName), indexMetadata);
  }

  @Override
  public String getIndexName(IndexSettings indexName) {
    return indexName.getResId();
  }

}
