package ch.unige.solidify.model;

import java.io.Serializable;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.unige.solidify.util.StringTool;

@Embeddable
public class RegularChangeInfo implements ChangeInfo, Serializable {

  private static final long serialVersionUID = -1153421579085880364L;

  @NotNull
  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = 3)
  private OffsetDateTime when;

  private String who;

  @Transient
  private String name;

  public RegularChangeInfo() {
    this.when = OffsetDateTime.now(ZoneOffset.UTC);
    this.who = "";
    this.name = "";
  }

  public RegularChangeInfo(OffsetDateTime changeTime) {
    this.when = changeTime;
    this.who = "";
    this.name = "";
  }

  public RegularChangeInfo(OffsetDateTime changeTime, String changeBy) {
    this.when = changeTime;
    this.who = changeBy;
  }

  public OffsetDateTime getWhen() {
    return this.when;
  }

  public String getWho() {
    return this.who;
  }

  public String getFullName() {
    if (StringTool.isNullOrEmpty(this.name)) {
      return this.getWho();
    } else {
      return this.name;
    }
  }

  public void setWhen(OffsetDateTime when) {
    this.when = when;
  }

  public void setWho(String who) {
    this.who = who;
  }

  public void setFullName(String fullName) {
    this.name = fullName;
  }

  @Override
  public String toString() {
    return "RegularChangeInfo{" +
            "when=" + this.when +
            ", who='" + this.who + '\'' +
            '}';
  }
}
