package ch.unige.solidify.model;

import static org.springframework.util.Assert.hasText;

import java.util.Objects;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import ch.unige.solidify.auth.model.ApplicationRole;
import ch.unige.solidify.rest.ResourceBase;
import ch.unige.solidify.rest.ResourceNormalized;
import ch.unige.solidify.rest.SolidifyModuleName;
import ch.unige.solidify.util.ReflectionTool;
import ch.unige.solidify.util.StringTool;

@Table(name = "ApplicationRole")
@Entity
public class SolidifyApplicationRole extends ResourceNormalized implements ApplicationRole {

  @NotNull
  @Column(unique = true)
  private String name;

  @NotNull
  @Column(unique = true)
  private int level;

  public SolidifyApplicationRole() {
  }

  public SolidifyApplicationRole(String name, int level) {
    hasText(name, "A role representation is required");
    this.setResId(name.toUpperCase());
    this.name = StringTool.toCamelCase(name);
    this.level = level;
  }

  public SolidifyApplicationRole(ApplicationRole applicationRole) {
    this.setResId(applicationRole.getResId());
    this.setName(applicationRole.getName());
    this.level = applicationRole.getLevel();
  }

  @Override
  public String getName() {
    return this.name;
  }

  @Override
  public int getLevel() {
    return this.level;
  }

  @Override
  public void init() {
    // Do nothing
  }

  @Override
  public String managedBy() {
    return SolidifyModuleName.ADMIN;
  }

  public void setName(String name) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), name);
  }

  @Override
  public String toString() {
    final StringBuilder builder = new StringBuilder();
    builder.append("Application Role [name=").append(this.name).append("]").append("[resId=")
            .append(this.getResId()).append("]");
    return builder.toString();
  }

  @Override
  public boolean update(ResourceBase item) {
    // Level cannot be updated
    if (((SolidifyApplicationRole) item).getLevel() != this.getLevel()) {
      return false;
    } else {
      return super.update(item);
    }
  }

  @Override
  public boolean equals(final Object o) {
    if (this == o)
      return true;
    if (!(o instanceof SolidifyApplicationRole))
      return false;
    final SolidifyApplicationRole that = (SolidifyApplicationRole) o;
    return this.getResId().equals(that.getResId());
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.name, this.level);
  }
}
