package ch.unige.solidify.rest;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import com.fasterxml.jackson.annotation.JsonIgnore;

/**
 * Interface used by Resources having sub resources
 */
public interface SubResourceContainer {

  <T> boolean addItem(T t);

  /**
   * Return a list of classes that are embedded sub Resources
   *
   * @return
   */
  @JsonIgnore
  default List<Class<? extends Resource>> getEmbeddedResourceTypes() {
    return new ArrayList<>();
  }

  default <T> List<String> getSubResourceIds(Class<T> type) {
    return null;
  }

  default <T> Page<String> getSubResourceIds(Class<T> type, Pageable pageable) {
    List<String> completeList = this.getSubResourceIds(type);
    long start = pageable.getOffset();
    long end = (start + pageable.getPageSize()) > completeList.size() ? completeList.size() : (start + pageable.getPageSize());
    if(start > end){
      return new PageImpl<>(Collections.emptyList(), pageable, 0);
    }
    return new PageImpl<>(completeList.subList((int)start, (int)end), pageable, completeList.size());
  }

  <T> boolean removeItem(T t);

}
