package ch.unige.solidify.business;

import java.net.URI;
import java.net.URISyntaxException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;

import ch.unige.solidify.OAIConstants;
import ch.unige.solidify.model.oai.OAISet;
import ch.unige.solidify.repository.OAISetRepository;
import ch.unige.solidify.service.ResourceService;
import ch.unige.solidify.specification.OAISetSpecification;
import ch.unige.solidify.util.StringTool;

@Service
public class OAISetService extends ResourceService<OAISet> {
  private static final Logger log = LoggerFactory.getLogger(OAISetService.class);

  private final OAISetRepository oaiSetRepository;

  public OAISetService(OAISetRepository oaiSetRepository) {
    this.oaiSetRepository = oaiSetRepository;
  }

  public OAISet findByName(String name) {
    return this.oaiSetRepository.findByName(name);
  }

  public OAISet findBySpec(String spec) {
    return this.oaiSetRepository.findBySpec(spec);
  }

  @Override
  public OAISetSpecification getSpecification(OAISet resource) {
    return new OAISetSpecification(resource);
  }

  public void initDefaultData() throws Exception {
    this.createOAISet(OAIConstants.OAI_DOI_REGISTRATION, "DOI Registration", "Set to registry DOIs", "*");
  }

  private void createOAISet(String spec, String name, String description, String query) {
    if (this.findByName(name) == null) {
      OAISet oaiSet = new OAISet();
      oaiSet.setSpec(spec);
      oaiSet.setName(name);
      oaiSet.setQuery(query);
      oaiSet.setEnabled(Boolean.TRUE);
      if (!StringTool.isNullOrEmpty(description)) {
        oaiSet.setDescription(description);
      }
      this.save(oaiSet);
      log.info("OAISet '{}' created", name);
    }
  }

  @Override
  protected void validateItemSpecificRules(OAISet oaiSet, BindingResult errors) {

    // Check if Spec is a valid URI
    try {
      new URI(oaiSet.getSpec());
    } catch (URISyntaxException e) {
      errors.addError(new FieldError(oaiSet.getClass().getSimpleName(), "spec", e.getMessage()));
    }

  }

}
