package ch.unige.solidify.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Files;
import java.nio.file.Path;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSResourceResolver;

import ch.unige.solidify.SolidifyConstants;

public class SchemaResourceResolver implements LSResourceResolver {
  private static final Logger logger = LoggerFactory.getLogger(SchemaResourceResolver.class);

  @Override
  public LSInput resolveResource(String type, String namespaceURI, String publicId, String systemId,
          String baseURI) {
    final LSInputImpl input = new LSInputImpl();

    // Find in package or in file system
    InputStream stream = this.getSchemaInputStream(systemId);
    if (stream == null) {
      final Path schemaDir = this.getSchemaPath(systemId);
      if (schemaDir == null) {
        return null;
      }
      try {
        stream = Files.newInputStream(schemaDir);
      } catch (final IOException e) {
        logger.trace("Unable to resolve resource", e);
        return null;
      }
    }

    input.setPublicId(publicId);
    input.setSystemId(systemId);
    input.setBaseURI(baseURI);
    input.setCharacterStream(new InputStreamReader(stream));
    return input;
  }

  private InputStream getSchemaInputStream(String systemId) {
    // Search in class path of the package (JAR or WAR)
    try {
      return new ClassPathResource(
              SolidifyConstants.SCHEMA_HOME + SolidifyConstants.URL_SEP + systemId)
                      .getInputStream();
    } catch (final IOException e) {
      logger.trace("Unable to get schema input stream", e);
      return null;
    }
  }

  private Path getSchemaPath(String systemId) {
    // Search in class path of the file system
    try {
      return new ClassPathResource(
              SolidifyConstants.SCHEMA_HOME + SolidifyConstants.URL_SEP + systemId).getFile()
                      .toPath();
    } catch (final IOException e) {
      logger.trace("Unable to get schema path", e);
      return null;
    }
  }
}
